@@ -69,8 +69,11 @@ class gpt:
         if system is None:
             system = self.system
 
+        use_system_property = False
+
         if system is not None:
             messages.insert(0, {'role': 'system', 'content': system})
+            use_system_property = True
         if user is not None:
             messages.append({'role': 'user', 'content': user})
 
@@ -99,6 +102,9 @@ class gpt:
             self._print_log('assistant', response.choices[0].message.content, '<>')
 
         self.previous = messages
+        if use_system_property:
+            self.previous = self.previous[1:]
+
         message = response.choices[0].message
         self.previous.append({'role':message.role,'content':message.content})
 
@@ -113,7 +119,14 @@ def reset(model='gpt-3.5-turbo'):
     staticGpt.previous = []
     staticGpt.model = model
 
-def print_messages(messages):
+def print_messages(conv, additional_messages=None):
+    if conv.system != None:
+        print('{SYS} ' + conv.system)
+
+    messages = conv.previous.copy()
+    if additional_messages != None:
+        messages += additional_messages
+
     for i in range(len(messages)):
         _print_message(message=messages[i], i=i)
 
@@ -146,6 +159,18 @@ async def _get_response(conv, prompt):
     for task in done:
         return task.result()
 
+def _get_boolean_input(message:str, default_value:bool):
+    while True:
+        arg = input(message)
+        if len(arg) == 0:
+            return default_value
+
+        arg = arg[0].lower()
+
+        if arg == 'n':
+            return False
+        if arg == 'y':
+            return True
 
 def conversation(model='gpt-3.5-turbo', system=None, messages=None, user=None, temperature=0, top_p=0, max_tokens=2048, frequency_penalty=0, presence_penalty=0):
     conv = gpt(model=model, system=system, temperature=temperature, top_p=top_p, max_tokens=max_tokens, frequency_penalty=frequency_penalty, presence_penalty=presence_penalty)
@@ -154,17 +179,14 @@ def conversation(model='gpt-3.5-turbo', system=None, messages=None, user=None, t
     
 
     if len(conv.previous) > 0 and conv.previous[0]['role'] == 'system':
-        system = conv.previous[0]['content']
+        conv.system = conv.previous[0]['content']
         conv.previous.pop(0)
-
-    if system != None:
-        print('{SYS} ' + system)
     
     def reprint_conversation(additional_message=None):
         os.system('cls' if (os.name == 'nt') else 'clear')
 
         print('Conversation started. Type ? for a list of commands.\n')
-        print_messages(messages=((conv.previous + [additional_message]) if additional_message != None else conv.previous))
+        print_messages(conv=conv, additional_messages=None if additional_message == None else [additional_message])
 
     reprint_conversation()
         
@@ -183,16 +205,17 @@ def conversation(model='gpt-3.5-turbo', system=None, messages=None, user=None, t
                 print('\t[!] Exit conversation')
                 print('\t[:] Run Python command')
                 print('\t[#] Set GPT\'s property')
+                print('\t[$] Set system message')
                 print('\t[+] Insert message before index (double + for assistant)')
-                print('\t[-] Remove message at index')
+                print('\t[-] Remove message at index (double - for clear conversation)')
                 print('\t[~] Change message at index (double ~ for reverse role)')
                 print('\t[&] Re-generate last GPT message')
-                print('\t[@] Copy last message to clipboard')
+                print('\t[@] Copy message to clipboard')
                 print('\t[@~] Copy code block to clipboard')
                 print('\t[@@] Copy conversation JSON to clipboard')
                 print('\t[] Reload conversation')
                 print('\t[\\] Override command')
-                print('\t[_] Multiline (Ctrl+X with Enter to exit)')
+                print('\t[_] Multiline (Ctrl+X with Enter to exit, Ctrl+C to cancel)')
                 continue
 
             
@@ -231,6 +254,14 @@ def conversation(model='gpt-3.5-turbo', system=None, messages=None, user=None, t
                         print(f'( Hint: Encapsulate strings in quotes (") )')
                     continue
 
+                elif prompt[0] == '$':
+                    message = prompt[1:] if len(prompt) > 1 else None
+
+                    conv.system = message
+
+                    reprint_conversation()
+                    continue
+
                 elif prompt[0] == '+':
                     space = prompt.find(' ')
                     is_assistant = len(prompt) > 1 and prompt[1] == '+'
@@ -259,7 +290,7 @@ def conversation(model='gpt-3.5-turbo', system=None, messages=None, user=None, t
                         conv.previous.pop(value)
                         reprint_conversation()
                     except (IndexError, ValueError):
-                        print(f'Error:\n\t`{prompt[1:]}` is not valid.')
+                        print(f'Error:\n\t`{prompt[1:]}` is not a valid index.')
                     continue
                 
 
@@ -303,27 +334,43 @@ def conversation(model='gpt-3.5-turbo', system=None, messages=None, user=None, t
                     prompt = ''
                     line_number = 1
 
-                    while True:
-                        new_line = input((8 - len(str(line_number))) * ' ' + str(line_number) + '> ')
-                        if new_line == '\x18':
-                            break
-                        if new_line == '\x15':
-                            previous_line = prompt[:-1].rfind('\n')
-                            if previous_line == -1:
-                                print('Error:\n\tNo previous line')
+                    try:
+                        while True:
+                            new_line = input((8 - len(str(line_number))) * ' ' + str(line_number) + '> ')
+                            if new_line == '\x18':
+                                break
+                            if new_line == '\x15':
+                                previous_line = prompt[:-1].rfind('\n')
+                                if previous_line == -1:
+                                    if prompt != '':
+                                        previous_line = prompt[:-1].find('\n')
+                                    else:
+                                        print('Error:\n\tNo previous line')
+                                        continue
+
+                                prompt = prompt[:(previous_line + 1)]
+                                line_number -= 1
+
+                                print('( Removed previous line )')
+
                                 continue
-
-                            prompt = prompt[:(previous_line + 1)]
-                            line_number -= 1
-
-                            print('( Removed previous line )')
-
-                            continue
-                        prompt += new_line + '\n'
-                        line_number += 1
+                            prompt += new_line + '\n'
+                            line_number += 1
+                    except KeyboardInterrupt:
+                        print()
+                        print('( Cancelled multi-line )')
+                        continue
 
                     prompt = prompt[:-1]
+
                     print('( Ended multi-line )')
+
+                    send_message = True
+                    if prompt == '':
+                        send_message = _get_boolean_input('Body is empty. Send anyways? (Y/n): ', True)
+
+                    if not send_message:
+                        continue
                 
                 elif prompt == '&':
                     if len(conv.previous) < 2 or conv.previous[-1]['role'] != 'assistant':
@@ -339,13 +386,14 @@ def conversation(model='gpt-3.5-turbo', system=None, messages=None, user=None, t
                     if not has_imported_pyperclip:
                         print('Error:\n\tThe pyperclip module is required to use clipboard\n\tInstall it using `pip install pyperclip`')
                         continue
-                    
-                    if prompt == '@':
-                        content = conv.previous[-1]['content']
+
+                    if prompt == '@@':
+                        content = json.dumps(conv.previous)
                         pyperclip.copy(content)
-                        print('( Copied last message to clipboard )')
+                        print('( Copied JSON to clipboard )')
                         continue
-                    if prompt[:2] == '@~':
+                    
+                    if len(prompt) >= 2 and prompt[:2] == '@~':
                         index = 0
                         try:
                             content = conv.previous[-1]['content']
@@ -364,37 +412,47 @@ def conversation(model='gpt-3.5-turbo', system=None, messages=None, user=None, t
                             print(f'Error:\n\tNo codeblock {index + 1}')
                         continue
 
-                    if prompt == '@@':
-                        content = json.dumps(conv.previous)
+                    if prompt[0] == '@':
+                        if len(conv.previous) == 0:
+                            print('Error:\n\tNo lines')
+                            continue
+
+                        index = -1
+                        try:
+                            if len(prompt) > 1:
+                                index = int(prompt[1:]) 
+                            content = conv.previous[index]['content']
+                        except (IndexError, ValueError):
+                            print(f'Error: `{prompt[1:]}` is not a valid index')
+                            continue
+
                         pyperclip.copy(content)
-                        print('( Copied JSON to clipboard )')
+                        if index == -1:
+                            print('( Copied last message to clipboard )')
+                        else:
+                            print(f'( Copied message {index} to clipboard )')
                         continue
+
+                    print('Error:\n\tInvalid @ command')
+                    continue
+
             else:
                 prompt = prompt[1:]
 
         cancel_sending = False
-        continue_loop = True
-        while continue_loop:
+        while True:
             try:
                 response = asyncio.run(_get_response(conv, prompt))
-                continue_loop = False
+                break
             except KeyboardInterrupt:
                 conv.previous = conv.previous[:-1]
                 arg = ''
-                while True:
-                    arg = input('Resend message? (Y/n): ')
-                    if len(arg) == 0:
-                        arg = 'y'
 
-                    arg = arg[0].lower()
+                if _get_boolean_input('Resend message? (Y/n): ', True):
+                    continue
 
-                    if arg == 'n':
-                        cancel_sending = True
-                        continue_loop = False
-                        break
-                    if arg == 'y':
-                        continue_loop = True
-                        break
+                cancel_sending = True
+                break
     
         if not cancel_sending:
             _print_message({'role':'assistant','content':response}, len(conv.previous) - 1)
